#include "finder.h"
#include "browser.h"

#include <QtDebug>
#include <QComboBox>
#include <QString>
#include <QSettings>
#include <QPair>
#include <QVector>
#include <QStringList>
#include <QWidget>
#include <QDialog>
#include <QLineEdit>

#define HISTORYLEN 12

Finder::Finder(QWidget *p) :  QDialog(p)
{
	ui.setupUi(this);
	connect(ui.pushButtonFind, SIGNAL(clicked() ), this, SLOT(FinderFind() ) );
	connect(ui.pushButtonCancel, SIGNAL(clicked() ), this, SLOT(FinderClose() ) );
	connect(ui.pushButtonClear, SIGNAL(clicked() ), this, SLOT(RemoveComboIcon() ) );
	connect(ui.radioButtonRegExp, SIGNAL(clicked() ), this, SLOT(SelectCurrentLineEdit() ) );
	connect(ui.radioButtonWildcards, SIGNAL(clicked() ), this, SLOT(SelectCurrentLineEdit() ) );
        ui.pushButtonClear->setIcon(QIcon(":icons/edit-clear-locationbar-rtl.png") );
        ui.pushButtonFind->setIcon(QIcon(":icons/find.png") );
        ui.pushButtonCancel->setIcon(QIcon(":icons/cancel.png") );
	connect(ui.checkBoxExes, SIGNAL(toggled(bool) ), this, SLOT(ShowSearchAmong(bool) ) );
	LoadHistory();
	/* NOTE: the regular expression radio button is already connected to the checkbox Case sensitive
	 * via the designer
	 */
	ui.checkBoxExes->setChecked(false);
	ui.groupBoxSearchMethod->setMaximumSize(QSize(340, 47) );
	ui.radioButtonWildcards->setChecked(true);
        ShowSearchAmong(false);
}

void Finder::FinderFind()
{
	expr = ui.comboBox->currentText();
	close();
}

void Finder::ClearLineEdit()
{
	QLineEdit* le = ui.comboBox->lineEdit();
	QString text = le->text();
	QStringList lsplit = text.split(" ");
	if(lsplit.size() > 0)
	{
		le->setText(lsplit[0] );
		le->setSelection(0, le->text().length() - 1);
	}
}

void Finder::SelectCurrentLineEdit()
{
	QLineEdit* le = ui.comboBox->lineEdit();
	le->selectAll();
}

void Finder::ShowSearchAmong(bool show) 
{
	ui.groupBoxSearchAmong->setHidden(!show);
	ui.radioButtonAll->setChecked(true);
	if(show)
	{
		setMinimumSize(QSize(331, 231) );
		setMaximumSize(QSize(331, 231) );
	}
	else
	{
		setMinimumSize(QSize(331, 180) );
		setMaximumSize(QSize(331, 180) );
	}

}

void Finder::FinderClose()
{
	expr = "";
	close();
}

void Finder::SaveHistory()
{
	QSettings s;
	int i;
	QStringList entries, results;
	for(i = pairs.size() - 1; i >= 0; i--)
	{
		entries << pairs[i].first;
		if(pairs[i].second == true)
			results << "true";
		else
			results << "false";
	}

	s.setValue("FinderHistory", QVariant(entries) );
	s.setValue("FinderResults", QVariant(results) );
}

void Finder::LoadHistory()
{
	QSettings s;
	int i;
	pairs.clear();
	QStringList history, hisresults;
	QVariant h, r;
	QLineEdit *linedit;
	QPair<QString, bool> newpair;
	h = s.value("FinderHistory");
	if( h.isValid() && h != QVariant() )
 		history = h.toStringList();
	else
		return;
	r = s.value("FinderResults");
	if(r.isValid() && r != QVariant() )
		hisresults = r.toStringList();
	else
		return;
	
	for(i = history.size() - 1; i >= 0; i--)
	{
		newpair.first = history[i];
		if(hisresults[i] == "true")
			newpair.second = true;
		else
			newpair.second = false;
		pairs.push_back(newpair);
	}
	for(i = history.size() - 1; i >= 0; i--)
	{
		if(pairs[i].second)
                    ui.comboBox->addItem(QIcon(":icons/find.png"), pairs[i].first );
		else
                    ui.comboBox->addItem(QIcon(":icons/warning.png"), pairs[i].first );
	}
	linedit = ui.comboBox->lineEdit();
	linedit->selectAll();
}

void Finder::AddToHistory(QString entry, bool found)
{
	QPair<QString, bool> newpair;
	newpair.first = entry;
	newpair.second = found;
	
	pairs.push_back(newpair);
	/* Remove the oldest entry if we reach the maximum size of the history */
	if(pairs.size() == HISTORYLEN)
		pairs.pop_front();
	SaveHistory();
}

void Finder::RemoveComboIcon()
{
//	ui.comboBox->setItemIcon(0, QIcon("") );
	ui.comboBox->clearEditText();
	ui.comboBox->setFocus();
}

Qt::CaseSensitivity Finder::CaseSensSearch()
{
	if(ui.checkBoxCS->isChecked() )
		return Qt::CaseSensitive;
	else
		return Qt::CaseInsensitive;

}
