/****************************************************************************
**
** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "mainwindow.h"
#include <QtDebug>
#include <QLockFile>
#include <cumacros.h>
#include <QApplication>
#include <QSystemTrayIcon>
#include <QCryptographicHash>
#include <QMessageBox>
#include <cupluginloader.h>
#include <quapplication.h>
#include <quappdbuscontroller.h>
#include "BrowserAdaptor.h"

#define CVSVERSION "$Name:  $"

void try_raise_running_instance() {
    CuPluginLoader pl;
    const char *dbus_plugin_nam = "libcumbia-dbus-plugin.so";
    QString plpa = pl.getPluginAbsoluteFilePath(QString(), dbus_plugin_nam);
    QPluginLoader pluginLoader(plpa);
    QObject *plugin = pluginLoader.instance();
    if (plugin) {
        QuDBusPluginInterface *dpi = qobject_cast<QuDBusPluginInterface *>(plugin);
        if(dpi) {
            QuAppDBusControllerInterface* m_qudbusctrl_i = dpi->getAppCtrlIface();
            printf("main.cpp: trying to find the running instance of %s\n", qstoc(qApp->arguments().join(" ")));
            QList<QuAppInfo> ail = m_qudbusctrl_i->findApps(qApp->arguments());
            if(ail.size() > 0) {
                printf("\e[1;32m (i) \e[0;32m main.cpp (qtbrowser)\e[0m: trying to bring %s to the foreground...\n",
                       qstoc(ail.first().dbusServiceName()));
                m_qudbusctrl_i->raise(ail.first());
            }
        }
        else {
            printf("qtbrowser: main.cpp: failed to cast plugin %s from %s\n", dbus_plugin_nam, qstoc(plpa));
        }
    }
    else {
        printf("qtbrowser: main.cpp: failed to load plugin %s from %s\n", dbus_plugin_nam, qstoc(plpa));
    }
}

int main(int argc, char **argv)
{
    QuApplication a(argc, argv);
    QString identifier;
    QString display;
    if(argc < 2) {
        QMessageBox::information(0, "Usage", QString("Usage: %1 path/to/config_file").arg(argv[0]));
        exit(EXIT_SUCCESS);
    }
    foreach(QString arg, qApp->arguments())
        identifier += arg;
    display = QString(getenv("DISPLAY"));
    identifier += display;

    QString tmpDir = QDir::tempPath();
    QString ide = qApp->arguments().at(1).section('/', -1).section('.', 0, 0);
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(identifier.toUtf8());
    QString lockFilenam = tmpDir + "/qtbrowser-" + ide + "-" + display + "-" + hash.result().toHex() + ".lock";
    QLockFile lockFile(lockFilenam);

    if(!lockFile.tryLock(100)){
        try_raise_running_instance();
        QMessageBox::information(0, "Browser already running", QString("The application %1 is already running")
                                 .arg(qApp->arguments().join(" ")));
        exit(EXIT_SUCCESS);
    }
    a.setApplicationName("Browser");
    QString version(CVSVERSION);
    a.setApplicationVersion(version);
    a.setProperty("author", "Giacomo");
    a.setProperty("mail", "giacomo.strangolino@elettra.eu");
    a.setProperty("phone", "375-8073");
    a.setProperty("office", "T2PT025");

    MainWindow *mainWin = new MainWindow(0);
    mainWin->setObjectName("BrowserMainWindow");
    mainWin->show();

    BrowserAdaptor *browserAdaptor = new BrowserAdaptor(mainWin);
    browserAdaptor->setObjectName("DbusBrowserAdaptor");

    int ret = a.exec();
    delete mainWin;
    return ret;
}


