#include "treestatemanager.h"
#include "browser.h"
#include <QtDebug>

TreeStateManager::TreeStateManager()
{

}

QString TreeStateManager::buildItemText(const QTreeWidgetItem *it) const
{
  QString text;
  while(it)
  {
        text = ("/" + it->text(0)) + text;
        it = it->parent();
  }
  text.remove(0, 1); /* remove first / */
  return text;
}

void TreeStateManager::m_getExpandedChildren(QTreeWidgetItem *parent)
{
  int i;
  bool hasExpanded = false;
  for(i = 0; i < parent->childCount(); i++)
  {
        QTreeWidgetItem *child = parent->child(i);
        if(child->isExpanded())
        {
          m_getExpandedChildren(child);
          hasExpanded = true;
        }
  }
  if(!hasExpanded && parent->isExpanded()) /* no children */
  {
// 	printf("m_getExpandedChildren: %s\n", qstoc(parent->text(0)));
        m_expandedItems << buildItemText(parent);
  }
}

void TreeStateManager::saveState(BrowserTree *tree)
{
  m_expandedItems.clear();

  QList<QTreeWidgetItem *> items = tree->findItems("*", Qt::MatchWildcard);
  foreach(QTreeWidgetItem *it, items)
  {
        if(it->isExpanded())
          m_getExpandedChildren(it);
  }
}

void TreeStateManager::restoreState(BrowserTree *tree)
{
  foreach(QString item, m_expandedItems)
  {
        QStringList pattern;
        if(item.count("/") > 0)
          pattern = item.split("/", QString::SkipEmptyParts);
        else
          pattern << item;

        QList<QTreeWidgetItem *> expItems = tree->findItems(pattern.first(), Qt::MatchExactly);
        foreach(QTreeWidgetItem *it, expItems)
        {
          if(pattern.size() > 1)
          {
                QTreeWidgetItem *lastExpChild = m_findItem(pattern.mid(1), it);
                if(lastExpChild)
                {
                  lastExpChild->setExpanded(true);
                  QTreeWidgetItem *par = lastExpChild->parent();
                  while(par)
                  {
                        par->setExpanded(true);
                        par = par->parent();
                  }
                }
          }
          if(it->text(0) == pattern.at(0))
                it->setExpanded(true);
        }
  }
}

QTreeWidgetItem *TreeStateManager::m_findItem(const QStringList& pattern,  QTreeWidgetItem *parent)
{
  QStringList copy = pattern;
  int i;
  QTreeWidgetItem *foundItem = NULL;
  for( i = 0; i < parent->childCount(); i++)
  {
        if(parent->child(i)->text(0) == pattern.first() && pattern.size() > 1)
          foundItem = m_findItem(copy.mid(1), parent->child(i));
        else if(parent->child(i)->text(0) == pattern.first() && pattern.size() == 1)
          foundItem = parent->child(i);
        if(foundItem)
          return foundItem;
  }
  if(parent->text(0) == pattern.first())
        return parent;
  if(i == 0 && pattern.size()) /* no children */
  {
        if(parent->text(0) == pattern.first())
          return parent;
  }
  else if(i == parent->childCount() && pattern.size())
  {
        if(parent->text(0) == pattern.first())
          return parent;
  }
//   printf("ritorno null per pattern / \"%s\" pattern.first() \"%s\" childCount %d patternSize %d i %d\n", qstoc(parent->text(0)),
// 		 qstoc(pattern.first()), parent->childCount(), pattern.size(), i);
  //qslisttoc(pattern);
  return foundItem;
}


